#!/usr/bin/perl

###############################################################################
# $Id$
#
# Copyright(c) 2006-2017 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#
# Retrieve current blocklist and store it in a file suitable for ipset.
# See HOWTO for more details
# This script is normally run once at setup and every two hours from cron
#
###############################################################################
use strict;
use warnings;
use diagnostics;
use File::Basename;
use Data::Dumper;

use FindBin;
use lib "$FindBin::Bin/lib/";
use ThreatStop;

########################################################################
##################### GLOBAL CONFIGURATION PART ########################
########################################################################

my $MYPATH  = $FindBin::Bin;
chomp (my $VERSION = `cat $MYPATH/VERSION`);
my $CONFILE = "${MYPATH}/threatstop.conf";
my $LOGGING = 1;
my $OUTFILE = "/tmp/tsoutput.txt";
my $TSSTR;
my $TSFILE = "TS.new";
my $LOGFILE= "ts-ipset.log";

my $DATA = {};
my $LCKFILE = $0 . ".lck";
my $CTIME;
my $ALLOW_IPSET_ARGS = ""; # gets filled in after config file is loaded (need allow_ipsetsize)
my $BLOCK_IPSET_ARGS = ""; # gets filled in after config file is loaded (need block_ipsetsize)

$DEBUG_LEVEL = 6 if ( defined($ARGV[0]) and $ARGV[0] eq "DEBUG");

#######################################################################
########################### FUNCTIONS PART #############################
########################################################################

####################################################
sub abortExec() {

        print "\n";
        print "Execution aborted....\n";
        exit 1;
}

##########################################################
#loadPrevConfData : Running from lib/Threatstop.pm module#
##########################################################
sub loadPrevConfData() {

	my $data_hash = {};
        my $err_msg;

	&printMsg("DEBUG","Previous configuration found ... Loading configuration data...");
        if ( &loadConfFileData($CONFILE,$data_hash, \$err_msg) != 0 ) {
               	&printMsg("ERROR","No configuration file has been found ...");
		return 1;
        } 

	#Now load retrived hash into our data holding hash
	foreach my $key ( keys %{$data_hash}) {
		$DATA->{$key} = $data_hash->{$key};
	}

	if ( ! defined $DATA->{'dns_server'} ) { 
		&printMsg("WARN","Missing dns_server parameter ... using default values...");
		$dns_server           = $DNS_LIST;
		$DATA->{'dns_server'} = $dns_server;
	} else {
		$dns_server = $DATA->{'dns_server'};
	}

	if ( ! defined $DATA->{'dig'} ) {
		$DATA->{'dig'} = '/usr/sbin/dig';
	}

	if ( ! defined $DATA->{'port'} ) {
		$DATA->{'port'} = 53;
	}

	return 0;
}

########################################################
#testingDNSConnection : test DNS accesaability Running #
#                       from lib/Threatstop.pm module  #
########################################################
sub testingDNSConnection() {

	my @dnsok;

	&printMsg("DEBUG","Testing DNS servers connection.");

        #Testing DNS servers access. Also verify dig command
        return 1 if ( &checkDNSaccess($DATA->{'dig'}, $DATA->{'dns_server'}, \@dnsok) != 0 );

	#Remove not responded servers from the list
	&printMsg("DEBUG","Updating DNS list with servers that respond.");
	$DATA->{'dns_server'} = join(' ',@dnsok);

        return 0;
}

############################################################################
#getLists: quiery DNS and upade $TSSTR setting content with retrieved data #
############################################################################
sub getLists() {

	my $list_name     = shift;
	my $list_type     = shift;
	my $max_size      = shift;
	my $ipset_args    = shift;
	my $count         = 0;

	my @lists;
	my $cidrs     = {};
	my @dns;

	&printMsg("DEBUG","Generating [$list_type] list.");

	#Take care of empty value
	unless ($list_name) {
	
		&printMsg("DEBUG","Creating $DATA->{'tsprefix'}${list_type}netnew as type nethash");
		my $newnet = "$DATA->{'tsprefix'}${list_type}netnew";
		$TSSTR .= "# List of ${list_type}ed network entries (none) for $newnet\n";
		$TSSTR .= "-N $newnet nethash $ipset_args\n";

		return 0;
	}

	@dns = split(' ',$DATA->{'dns_server'});

	#Go on and build the list
	my $cmd = "$DATA->{'dig'} -p $DATA->{'port'} ptr \@$dns[0] $list_name";
	my $res = qx{$cmd};

	&printMsg("DEBUG","$cmd");

	if ( $res !~ m/opcode: QUERY, status: NOERROR,.*QUERY: 1, ANSWER: (\d+)/s ) {
		&printMsg("ERROR","Initial dig failed.");
		return 1;
	}

	while ( $res =~ m/\.\s+\d+\s+IN\s+PTR\s+(\S+)\.\s/sg) {
		push(@lists, $1);
	}

	for my $l (@lists) { 
		&printMsg("DEBUG","$DATA->{'dig'} -p $DATA->{'port'} apl \@$dns[0] $l");
		$cmd = "$DATA->{'dig'} -p $DATA->{'port'} apl \@$dns[0] $l";
		$res = qx{$cmd};
		if ( $res !~ m/opcode: QUERY, status: NOERROR,.*QUERY: 1, ANSWER:\s+(\d+)/s ) {
			&printMsg('ERROR',"Dig $l failed with error [$res]");
			return 1;
		}

		&printMsg("DEBUG","$res");

		while ( $res =~ /$l\.\s+\d+\s+IN\s+APL\s+\d:(\d+\.\d+\.\d+\.\d+)\/(\d+)/sg) {

			#If max size threshhold is reached, skip remaining entries
			last if ( $count >= $max_size );

			# GS-1368 : all go into a hethash ipset
			if ( ! defined $cidrs->{$1} ) {

				# not yet tracked
				$cidrs->{$1}=$2;
				$count++;
			} elsif ( $2 < $cidrs->{$1} ) {

				# already tracked, but found a wider netmask
				# (so upon replacing, no need to increment counter)
				&printMsg("DEBUG","Wider netmask [$2] encountered for ip [$1]");
				$cidrs->{$1}=$2;
			}
		}

		#If max size threshhold is reached, skip remaining entries
		if ( $count >= $max_size ) {
			&printMsg("INFO","Maximum ${list_type} items reached [$max_size]... skipping remaining items.");
			last;
		}
	}

	my $cidrs_size = scalar keys %$cidrs;

	&printMsg('DEBUG',"Total ${list_type} networks: $cidrs_size");
	
	#Updating TS string with list data
	$TSSTR .= "# List of ${list_type}ed network entries\n";
	if ( $DATA->{'ipsetver'} ne '6.0') {
		$TSSTR .= "-N $DATA->{'tsprefix'}${list_type}netnew nethash $ipset_args\n";
	}
	$TSSTR .= join ('',map {"-A $DATA->{'tsprefix'}${list_type}netnew $_/$cidrs->{$_}\n"} keys %$cidrs);
	
	return 0;
}

####################################################
sub generateLists() {

	my $t;	

	&printMsg("DEBUG","Preparing to generate lists.");

	#Generate TS string for previous versions
	&getCurrentTime(\$t);

	$TSSTR  = "# Generated by vyattaipset $VERSION\n";
	$TSSTR .= "# Process started at $t\n";
	if ( $DATA->{'ipsetver'} eq '6.0' ) {
		$TSSTR .= "# Create sets\n";
		$TSSTR .= "-N $DATA->{'tsprefix'}blocknetnew nethash $BLOCK_IPSET_ARGS\n"; 
		$TSSTR .= "-N $DATA->{'tsprefix'}allownetnew nethash $ALLOW_IPSET_ARGS\n"; 
	}

	#Run generate lists code
	return 1 if ( &getLists($DATA->{'allow_list'}, 'allow', $DATA->{allow_ipsetsize}, $ALLOW_IPSET_ARGS) != 0 );
	return 1 if ( &getLists($DATA->{'block_list'}, 'block', $DATA->{block_ipsetsize}, $BLOCK_IPSET_ARGS) != 0 );

	$t=gmtime;
	$TSSTR .=($DATA->{'ipsetver'} eq '6.0'?'':"COMMIT\n")."\n# Completed on $t\n";

	return 0;
}

########################################
#safeGeneratedData : TSFILE = TS.new   #
########################################
sub safeGeneratedData() {

	&printMsg("DEBUG","Saving generated lists data.");

	#Ready to upoad generates TS string into file
	eval {
		open (FILE, '>' . $MYPATH . '/' . $TSFILE) or die $!;
	};

	if ( $@ ) {
		&printMsg("ERROR","Creating TS lists file failed with error [$@]");
		return 1;
	}

	print FILE $TSSTR;
	close(FILE);

	return 0;
}

##############################################################
#setApply: runs ipsetapply.sh script which loads data stored #
#          in TS.new file into device                        #
##############################################################
sub setApply() {
	
	my $rc;
	my $rv;
	my $cmd;

	&printMsg("DEBUG","Running ipsetapply script.");

	$cmd = "${MYPATH}/ipsetapply.sh 2>&1";
	&printMsg("DEBUG","Running $cmd");

	$rv=qx($cmd);
	$rc = $?;

	print "$rv";

	return 1 if ( $rc != 0 );
	return 0;
}

#Edit log rotate files in place to add in our logupload client as a postrotate script.
#on vyatta.
sub fixLogRotateConf {
    my $FILE;
    my $script;
    $script = $FindBin::Bin . "/loguploadclient.pl";
    if ( -w "/etc/logrotate.d/threatstop.log" ) {
        $FILE = "/etc/logrotate.d/threatstop.log";
    }
    elsif ( -w "/opt/vyatta/etc/logrotate/file_threatstop.log" ) {
        $FILE = "/opt/vyatta/etc/logrotate/file_threatstop.log";
    }
    else {
	&printMsg("ERROR","couldn't find logrotate config file");
        return 0;
    }
    open(my $fh,"+<",$FILE) or die "Can't open $FILE for read+write\n";
    my $out = '';
    my $postrotate = 0;
    my $logupload = 0;
    while (<$fh>) {
        $postrotate = 1 if m/postrotate/;
        $logupload = 1 if m/$script/;
    }
    #old logrotate files with postrotate included already
    if ($postrotate and not $logupload) {
	&printMsg("DEBUG","adding '$script' to postrotate section");
        seek($fh,0,0);
        my $inpostrotate = 0;
        while (<$fh>) {
            if ( m/postrotate/ ) {
                $inpostrotate = 1;
                $out .= "  postrotate\n";
                next;
            }
            if ( $inpostrotate && m/endscript/ ) {
                $out .= "    $script\n";
                $out .= "  endscript\n";
                $inpostrotate = 0;
                next;
            } else {
                $out .= $_;
            }
        }
        seek($fh,0,0);
        print $fh $out;
        close($fh);
    }
    # new logrotate files without a postrotate script.
    elsif ( not $logupload ) {
	&printMsg("DEBUG","adding postrotate section with '$script'");
        seek($fh,0,0);
        while (<$fh>) {
            if ( m/\}/ ) {
                $out .= "  postrotate\n";
                $out .= "    $script\n";
                $out .= "  endscript\n";
                $out .= "\}\n";
                last;
            } else {
                $out .= $_;
            }
        }
        seek($fh,0,0);
        print $fh $out;
        close($fh);
    }
}
   


########################################################################
########################## MAIN PART ###################################
########################################################################

#system("/usr/bin/clear");

print "\n";

fixLogRotateConf();  # Make sure logupload script is always set in logrotate file. 
&abortExec() if ( &setWorkingDir()        != 0 );    # Switch to working directory
&logInit($LOGFILE);

&getCurrentTime(\$CTIME);
&printMsg("INFO","Starting ThreatSTOP setup operation v$VERSION at $CTIME");

&setLock();

&abortExec() if ( &loadPrevConfData()     != 0 );    # Load previously stored configuration
$ALLOW_IPSET_ARGS = "--hashsize 32768 --maxelem $DATA->{'allow_ipsetsize'}"; # had to wait until config was loaded
$BLOCK_IPSET_ARGS = "--hashsize 32768 --maxelem $DATA->{'block_ipsetsize'}"; # had to wait until config was loaded
&abortExec() if ( &generateLists()        != 0 );    # Generate lists. Actual work is done here
&abortExec() if ( &safeGeneratedData()    != 0 );    # Store generates data
&abortExec() if ( &setApply()		  != 0 );    # Apply new data

print "\n";
&getCurrentTime(\$CTIME);
&printMsg("INFO","Finished ThreatSTOP ipsetget operation v$VERSION at $CTIME");

END {
        &setUnLock(); 
	qx(rm -f $LCKFILE) if ( -e $LCKFILE );
}


