#!/bin/bash
#
# Copyright(c) 2006-2017 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

###########################################################
################### GLOBAL VARIABLES PART #################
###########################################################

LOCKFILE="lock.lck"
CONFIG="threatstop.conf"
IPSETTOOL="./ipsetapply.sh"
IPSETRULES="TS.org"
MYUID=$(id -u)
OS_VER_TRIGGER="6.5"
IS_ERRS=0

DEBUG="false"
[ "$1" = "DEBUG" ] && DEBUG="true"

###########################################################
############### FUNCTIONS PART ############################
###########################################################
#make sure we are group vyattacfg this may cause us to exec ourselves with sg
#setGID must be called with $@ as an argument otherwise command line options are lost
function setGID() {
	local GRP=$(id -gn)
    local uid=$(id -u)
    if [ "$uid" == "0" ]; then
        echo "Refusing to run as root."
        exit 1
    fi
	if [ "$GRP" != "vyattacfg" ]; then
		echo "Setting our group to vyattacfg."
        umask 002
		exec sg vyattacfg "$0 $*"
	fi
}
############################################
function echoMsg() {

        case $1 in
                ERROR) echo "[ERROR] : $2"
                        ;;
                WARN)  echo "[WARN ] : $2"
                        ;;
                ALERT) echo "[ALERT] : $2"
                        ;;
                INFO)  echo "[INFO ] : $2"
                        ;;
                DEBUG) if [ "$DEBUG" = true ]; then
                                echo -e "\t[DEBUG] : $2"
                       fi
                        ;;
                *) echo "SCRIPT ERROR : Unrecognized parameter [$1]"
        esac
}

############################################
function exitting() {

        echo
        echo "*****************************************************"
        echo "*Unrecoverable errors encountered ... Aborting...   *"
        echo "*Please fix, then run script again.                 *"
        echo "*****************************************************"
        echo
        exit 1
}

############################################
setLock() {

        trap "/bin/rm -f $LOCKFILE; return 1" 0 1 2 3 15

        echoMsg DEBUG "Locking current instance."

        if [ -e $LOCKFILE ]; then
                ps=$(cat $LOCKFILE)
                echoMsg ERROR "Failed to lock ... Another instance [$ps] of the same script is already running"
                return 1
        fi

        echo $$ > $LOCKFILE
        return 0
}

############################################
function unLock() {

        echoMsg DEBUG "Unlocking current instance."
        /bin/rm -f $LOCKFILE
        return 0
}

############################################
function setWorkingDir() {

	echoMsg DEBUG "Setting working directory."

	local mydir=$(dirname $0)

	cd $mydir
	[ $? -ne 0 ] && return 1
	return 0
}

############################################
function verifyUserID() {

        echoMsg DEBUG "Verifying user credentials."
        return 0
}

###########################################
function loadConfig() {

        echoMsg DEBUG "Loading configurations."

        if [ ! -f $CONFIG ]; then
                        echoMsg ERROR "Cannot find config file: \"$CONFIG\""
                        return 1
        fi
        . $CONFIG

        return 0
}

#############################################################
#unsetRules : runs script to revert rules to origonal state #
# This is done by running configure.router|bridge.sh revert #
#############################################################             
function unsetRules() {

	echoMsg DEBUG "Starting unset rules operation."

	for i in configure.route.sh configure.bridge.sh
	do
		if [ -e $i ]; then
			./$i revert
			mv $i $i.old
		fi
	done	
	
	echoMsg DEBUG "Attempting to remove any existing ipsetrules."	
	for i in blocknet allownet
	do  
		if sudo ipset -L | grep $tsprefix$i >/dev/null 2>&1; then
			rc=$(sudo ipset -X $tsprefix$i)
			rv=$?
			if [ $rv -ne 0 ]; then
				echoMsg ERROR "Failed removing $tsprefix$i  You may want to try removing it manually."
				echoMsg ERROR "Delete $tsprefix$i from any existing firewall rules and then"
				echoMsg ERROR "delete $tsprefix from firewall group in configure mode to get rid of it."
				echoMsg ERROR "Then run this script again."
			fi  
		fi  
	done

	return 0
}

##################################################################
#detectVersion : detects current OS version. Needed for relevant #
#                conf file removal                               #
##################################################################
function detectVersion() {

	OS_FULL=$(vbash -ic "show version | grep Version | cut -d\: -f2 | sed 's/ *//'")
	OS=$(echo $OS_FULL | sed 's/VC\([0-9]\+\.[0-9]\+\).*/\1/')

	return 0
}

#############################################################
#cleanCronTab : restores original crontab                   #
#############################################################
function cleanCronTab() {

	echoMsg DEBUG "Cleaning up crontab entries."

	if sudo crontab -l 2>&1 | grep "no crontab" &>/dev/null; then
		echoMsg DEBUG "No configured crontab detected ... Skipping to the next stage."
		return 0
	fi 

	sudo crontab -l | sed -e '/ThreatSTOP/d'                                   \
			 -e '/[0-9]\+:[0-9]\+, [0-9]\+:[0-9]\+, [0-9]\+:[0-9]\+/d' \
                         -e '/ipsetget/d'                                          \
			 -e '/exec.*logrotate.*stat.*7.*100000/d'                  \
                         -e '/Force a logrotate.*minutes after the hour/d'         \
                         -e '/threatstop/d'                                        \
                         -e '/Update services host/d'                              \
                         -e '/update_services\.pl/d' > crontab.org

	sudo crontab -r
	sudo crontab crontab.org

	return 0
}

#############################################################
#cleanRClocal : removes threatstop entries from rc.local    #
#############################################################
function cleanRClocal() {

	echoMsg DEBUG "Cleaning up rc.local entries."

	cat /etc/rc.local | sed '/ipsetget/d' > rc.local.org
	rv=$(sudo mv -f rc.local.org /etc/rc.local 2>&1)
	if [ $? -ne 0 ]; then
		echoMsg ERROR "Restoring messages file failed with error [$rv]"
		IS_ERRS=1
		return 0
	fi	
 
	return 0
}

#############################################################
function cleanLogrotationOld() {

	echoMsg DEBUG "Cleaning up messages logrotation file."

	if [ ! -e logrotated.messages.org ]; then
		echoMsg ERROR "Can not find a previous version of messages logrotation file... Skipping restoration..."
		IS_ERRS=1
		return 0
	fi

	rv=$(cp -v logrotated.messages.org /etc/logrotate.d/messages 2>&1)
	if [ $? -ne 0 ]; then
		echoMsg ERROR "Restoring messages file failed with error [$rv]"
		IS_ERRS=1
		return 0
	fi	

	return 0
}

#############################################################
function cleanLogrotationNew() {

	echoMsg DEBUG "Cleaning up threatstop-logrotate.conf file."
	sudo rm -f "/etc/logrotate.d/threatstop-logrotate.conf"
	
	echoMsg DEBUG "Cleaning up 01-threatstop-rsyslog.conf file."
	sudo rm -f "/etc/rsyslog.d/01-threatstop-rsyslog.conf"

	return 0
}

#############################################################
#cleanLogrotation : clean up logrotation mechanism          #
#############################################################
function cleanLogrotation() {

#	VER=$(vbash -ic "show version |                 \
#                         grep Version |                 \
#                         cut -d\: -f2 | sed 's/ *//'" | \
#			 sed 's/[A-Z]\+\([0-9]\+\.[0-9]\+\).*/\1/')
#
#	ver_trigger=$(echo "$VER $OS_VER_TRIGGER" | awk '{if ($1 < $2) print "true"; else print "false"}')
#	if [ "$ver_trigger" == "true" ]; then
#		cleanLogrotationOld
#		[ $? -ne 0 ] && return 1
#	else 
#		cleanLogrotationNew
#		[ $? -ne 0 ] && return 1
#	fi

	if vbash -ic 'show configuration' | grep threatstop.log >/dev/null 2>&1; then
		/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin || return 1
		/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete system syslog file threatstop.log || return 1
		/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit || return 1
		/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper save || return 1
		/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper end || return 1
	fi

	return 0
}


###########################################################
##################### MAIN PART ###########################
###########################################################

clear
setGID $@
echo
echo "*****************************************************"
echo "*      Starting ThreatStop uninstall operation      *"
echo "*****************************************************"
echo

#setLock
#[ $? -ne 0 ] && exitting
setWorkingDir
[ $? -ne 0 ] && exitting

verifyUserID
[ $? -ne 0 ] && exitting

loadConfig
[ $? -ne 0 ] && exitting

cleanCronTab
[ $? -ne 0 ] && exitting

cleanRClocal
[ $? -ne 0 ] && exitting

cleanLogrotation
[ $? -ne 0 ] && exitting

unsetRules
[ $? -ne 0 ] && exitting

#unLock
#[ $? -ne 0 ] && exitting

echo
echo "****************************************************"
echo "* ThreatSTOP has been uninstalled from your system *"
echo "****************************************************"
echo
