#!/usr/bin/perl
###############################################################################
# $Id$
#
# Copyright(c) 2006-2017 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#
# ThreatSTOP Vyatta install script 
# See HOWTOs for detailed instructions. 
# Run once with B or R as install type to setup the rules
# Run if you change firewall name / block lists or add/remove allow lists with 
# the U install type
# NB 1) if you just change content the lists used for this firewall there is 
# no need to run again 
# 2) If you need to edit, revert or delete the firewall config you may need
# to run
#       ipsetapply.sh TS.org 
# first to make sure that the original ipset sets are in place
# minor bug fixes from 0.8 and add logging via logupdate.pl.
# 0.91 modify router code so that it only asks for the external i/f and applies 
# all rules to/from it. Add log upload support in threatstop.conf
# 0.92 typos fixed. Modify ipsetapply.sh to correctly restore when it auto sudos
# 0.99 release candidate - add logrotate logic and modify crontab appropriately
# and also improve messages
# 1.00 (probably should not have jumped to 0.99) two fixes
# first fix lograte cron task (add path to logrotate i.e. /usr/sbin/logrotate)
# second fix to use existing firewall name if it exists. ThreatSTOP rules go in
# at the place specified by the installer
# 1.01 fix permissions bug reported by Stig Thormodsrud 
# (add sg to vyatta-cfg-cmd-wrapper lines)
# 1.02 fix log upload related bug reported by Stig Thormodsrud 
# (add /usr/sbin to front of 'invoke-rc.d' in /etc/logrotate.d/messages)
# also change log upload behaviour s.t. it runs postrotate not prerotate
# 1.03 add list of available interfaces when asking which one to use and option
# external-type for those cases where it isn't ethernet
# also detect vyatta 6.3/ipset 6.0 and set version appropriately in threatstop.comf
#
###############################################################################
use strict;
use warnings;
use diagnostics;
use Cwd;
use Cwd qw(abs_path);
use File::Basename;
use File::Copy;
use Getopt::Long;
use Switch;
use POSIX qw(getgrnam setgid);
use FindBin;
use lib "$FindBin::Bin/lib/";
use ThreatStop;

########################################################################
##################### GLOBAL CONFIGURATION PART ########################
########################################################################

my $MYDIR = "$FindBin::Bin";

chomp (my $VERSION=`cat $MYDIR/VERSION`);
my ($CTIME,$ETIME);
my $START_EPOCH = time();

my $UID  = $<;
my $OPTS = {};
my $SYSTEM = {};

my $BLOCK_LIST     = 'basic.threatstop.local';
my $ALLOW_LIST     = 'dns.threatstop.local';
my $URL            = 'https://threatstop.com/cgi-bin/logupload.pl';
my $LOGFILE        = '/var/log/user/threatstop.log.1';
my $LOGSYSFILEOLD  = '/etc/logrotate.d/messages';
my $LOGALTFILE     = '/var/log/user/threatstop.log.1';
my $LOGSYSFILE     = '/etc/logrotate.d/threatstop-logrotate.conf';
my $LOGRSYSLOG     = '/etc/rsyslog.d/01-threatstop-rsyslog.conf';
my $PREFIX         = 'TS';
my $TYPE           = 'u';
my $CONFILE        = $MYDIR . "/threatstop.conf";
my $DEFAULTCONFILE = $MYDIR . "/threatstop.conf.tpl";
my $CLEANUP_SCRIPT = $MYDIR . "/revert.sh";
my $UPDATE_SVCS_SCRIPT = $MYDIR . "/update_services.pl";
my $PORT           = 53;
my $MAXPOLICYSIZE  = 64 * 1024;   # default
my $MAXPOLICYALLOW = 1024 * 1024; # absolute max
my $ALLOW_IPSETPERCENTAGE = 20;
my $ALLOW_MINIPSETSIZE = 2500;

my $TMP_LOGFILE1 = 'logrotated.messages.tmp';
my $TMP_LOGFILE2 = '01-threatstop-rsyslog.conf.tmp';
my $TMP_LOGFILE3 = 'threatstop-logrotate.conf.tmp';

my $ORGLOGMSG;
my $ORGRCLOCAL;
my $APPLY      = 0;
my $OS_VER_TRIGGER = 6.5;
my $OS_VER_TRIGGER_OLDER = 6.2;

#my $LCKFILE = $0 . ".lck";

my $SETUPLOG   = $MYDIR . "/tsinstall.log";
# DEBUG_LEVEL is imported from ThreatStop module; only need to adjust it here
$DEBUG_LEVEL = 6 if ( $ARGV[0] eq "DEBUG");

########################################################################
########################### FUNCTIONS PART #############################
########################################################################

####################################################
sub printMsg1() {

	return 0 if ( defined $OPTS->{'silent'} );

	print "\n";
	print "===========================================================================\n";
	print "If you have not specified setup options on the command line then you will be\n";
	print "given the chance to specify them now. First time users running this by pasting\n";
	print "in the command from the ThreatSTOP website should probably not change anything\n";
	print "except the firewall names and start rule number if you have already created\n";
	print "some firewall rules.\n";
	print "\n";
	print "On subsequent runs, probably the only things to change will be the block and\n";
	print "allow list ids. For each option, the default value is specified in [], just\n";
	print "press the ENTER key to accept it.\n";
	print "\n";
	print "Note for the paranoid. The proposed changes to the Vyatta config, the changes\n";
	print "to /etc/rc.local, /etc/logrotate.d/messages and the new crontab are created in\n";
	print "the installation directory. If you choose not to allow this script to apply the \n";
	print "changes automatically then you can review them and then apply them manually.\n";
	print "===========================================================================\n";
	print "\n";

	return 0;
}

####################################################
sub printMsg2() {

	return 0 if ( defined $OPTS->{'silent'} );

	print "\n";
	print "===========================================================================\n";
	print "If you did not allow the setup program to automatically change the system\n";
	print "configuration you will need to do this now.  Refer to the manual to see what\n";
	print "needs to be done but in general you will need to run the config script if one\n";
	print "was created (config.*.sh), add the created 'crontab' file to the crontab for\n";
	print "root and, if this is not an upgrade, copy the modified rc.local to /etc and\n";
	print "the file logrotated.messages to /etc/logrotate.d/messages\n";
	print "\n";
	print "Finally if you did not run it, you will need to run ipsetget.pl to download\n";
	print "and install the current block and allow lists\n";
	print "===========================================================================\n";
	print "\n";

	return 0;
}

#switch our primay group to vyattacfg so bad things do not happen when we execute the cfg-cmd-wrapper
#also set up our umask to a value that won't cause vyattacfg system to choke 
#(all files that get created through the cfg-cmd-wrapper need to be group writable by vyattacfg or bad things happen)
#in order to do this as a non-priveleged process we re-exec ourselves with sg
sub setGID {
	my $target_gid = getgrnam('vyattacfg') or die "Can't find vyattacfg group.";
	my @current_gids = split( /\s+/, $( );
	if ( ! grep( /^$target_gid$/,  @current_gids ) ) {
		&printMsg("DEBUG","Changing group to vyattacfg\n");
		exec "sg vyattacfg \"$0 @ARGV\""
	}
	&printMsg("DEBUG","running under group vyattacfg\n");
	umask 0002;
	return 0;
}

###############################################################
#abortExec : when aborting installation we also want to clean #
#            up tmp files we have created                     #
###############################################################
sub abortExec() {

	&printMsg("DEBUG", "Cleaning up installed settings..."); 

	#First cleanup already installed settings
	if ( -e "$CLEANUP_SCRIPT" ) {
		&printMsg("DEBUG","Running $CLEANUP_SCRIPT");
		qx($CLEANUP_SCRIPT);
	}

	#Now remove possible tmp leftover files 
	if ( -e "$OPTS->{'installdir'}/logrotated.messages" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/logrotated.messages");
		qx(suo rm -f $OPTS->{'installdir'}/logrotated.messages);
	}

	if ( -e "$OPTS->{'installdir'}/01-threatstop-rsyslog.conf.tmp" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/01-threatstop-rsyslog.conf.tmp");
		qx(sudo rm -f $OPTS->{'installdir'}/01-threatstop-rsyslog.conf.tmp); 
	}

	if ( -e "$OPTS->{'installdir'}/threatstop-logrotate.conf.tmp" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/threatstop-logrotate.conf.tmp");
		qx(sudo rm -f $OPTS->{'installdir'}/threatstop-logrotate.conf.tmp); 
	}

	if ( -e "$OPTS->{'installdir'}/crontab" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/crontab");
		qx(sudo rm -f $OPTS->{'installdir'}/crontab); 
	}

	if ( -e "$OPTS->{'installdir'}/crontab.org" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/crontab.org");
		qx(sudo rm -f $OPTS->{'installdir'}/crontab.org); 
	}

	if ( -e "$OPTS->{'installdir'}/rc.local" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/rc.local");
		qx(sudo rm -f $OPTS->{'installdir'}/rc.local); 
	}

	if ( -e "$OPTS->{'installdir'}/rc.local.org" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/rc.local.org");
		qx(sudo rm -f $OPTS->{'installdir'}/rc.local.org); 
	}

	if ( -e "$OPTS->{'installdir'}/configure.route.sh" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/configure.route.sh");
		qx(sudo rm -f $OPTS->{'installdir'}/configure.route.sh); 
	}

	if ( -e "$OPTS->{'installdir'}/configure.bridge.sh" ) {
		&printMsg("DEBUG","Removing $OPTS->{'installdir'}/configure.bridge.sh");
		qx(sudo rm -f $OPTS->{'installdir'}/configure.bridge.sh); 
	}

	

	print "\n";
	print "Execution aborted....\n";
	exit 1;
}

####################################################
sub verifyUID() {
    if ($UID == 0) {
        print "Running this script as root is not supported.\n";
        exit(1);
    }
}

####################################################
sub ipSetVerDetector() {

	my $ver;
	my $rc;

	$OPTS->{'ipset'} = qx(sudo which ipset); 
	chomp ($OPTS->{'ipset'});
	&printMsg("DEBUG","ipset location: $OPTS->{'ipset'}\n");
	
	$ver =  qx($OPTS->{'ipset'} -V | head -n1 2>/dev/null);
	$rc = $?;

	$ver =~ m/ipset v(\d+\.\d+)/;
	$ver = 0 if ( ! defined $ver or $ver eq '' or $rc != 0);

	#chomp($ver);
	$OPTS->{'ipsetver'} = $1;
	return 0;
} 

####################################################
#ipSetVerDetector : detects Vyatta OS version      #
####################################################
sub OSVerDetector() {
	
	my ($os_ver_full,$os_ver);
	my $hw_model;

	#First see if we are an EdgeMax router. If yes than use V6.5 version settings
	$hw_model = qx(vbash -ic "show version | grep EdgeRouter");
	if ( defined $hw_model and $hw_model ne "" ) {
		$os_ver = 6.5;
		$os_ver_full = "EdgeMax";
		$OPTS->{'osver'}     = $os_ver;
		$OPTS->{'osverfull'} = $os_ver_full;
		return 0;
	} 
        $hw_model = qx(vbash -ic "show version | grep VyOS");
	if (defined $hw_model and $hw_model ne "" ) {
		$os_ver = 6.7;
		$os_ver_full = 'VyOS';
		$OPTS->{'osver'}     = $os_ver;
		$OPTS->{'osverfull'} = $os_ver_full;
		return 0;
	}

	
	#See if we are running newer Brocade version
	$hw_model = qx(vbash -ic "show version | grep Brocade");
	$os_ver_full = qx(vbash -ic "show version | grep Version | cut -d\: -f2 | sed 's/ *//'");
    if ($hw_model  =~ m/vRouter/) {
        if ( $hw_model =~ m/ 54\d\d/) {
            #5400 series detected do nothing.
        }
        if ( $hw_model =~ m/ 56\d\d/) {
            &printMsg("ERROR","Brocade vRouter 5600 is not currently supported.  Please contact support.");
            $OPTS->{'osver'} = "6.7";
            exit(1);
        }
    }
	if ( defined $hw_model and $hw_model ne "" ) { 
		$os_ver_full =~ /(VSE)([0-9]+\.[0-9]+)(.*)/;
		$os_ver = $2;
		chomp($os_ver_full);
	} else {
		#Standard Vyatta handling
		$os_ver_full =~ /(VC)([0-9]+\.[0-9]+)(.*)/;
		$os_ver = $2;
		chomp($os_ver_full);
	}

	$OPTS->{'osver'}     = $os_ver;
	$OPTS->{'osverfull'} = $os_ver_full;

	return 0;
}

####################################################
# rulenamestart :                                  #
####################################################
sub rulenamestart {

        my ($intf, $dir) = @_;
	my $prefix=($OPTS->{'type'} eq 'b')?'bridge':'rt';
        my ($firstrule, $defaultaction);

	print "\nFirewall name for interface $intf direction $dir: [".$OPTS->{'setprefix'}.$prefix.$dir."rule] ";
	chomp($OPTS->{$prefix.$dir.'rule'} = <STDIN>);
	$OPTS->{$prefix.$dir.'rule'} = $OPTS->{'setprefix'}.$prefix.$dir.'rule' unless ($OPTS->{$prefix.$dir.'rule'});

	while (1) {
		print "\nInsert ThreatSTOP rules beginning at number?: [10] ";
		chomp($firstrule = <STDIN>);

		if ( $firstrule eq "" ) { $firstrule = 10; }
		elsif ( $firstrule =~ /^\s*(\d+)\s*$/ && $1 > 0 ) {
			$firstrule = $1;
		}
		else {
			print "Must be a positive numeric value.\n";
			next;
		}
		
		$OPTS->{$prefix.$dir.'rule'.$_} = $firstrule+$_ for (0..(($OPTS->{'type'} eq 'b')?7:3) );
		last;
	}

	print "\nAdd default accept?: (strongly recommended if you do not have other rules for\nthis firewall name, not otherwise) [Y] ";
        chomp($defaultaction = lc <STDIN>);
        $defaultaction = 'y' unless $defaultaction;

	$OPTS->{$prefix.$dir.'rule-default'} = ($defaultaction eq 'y')?
        q(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' ').$OPTS->{$prefix.$dir.'rule'}.q(' 'default-action' 'accept'):'';

	return 0;
}

####################################################
sub usage() {

	system("/usr/bin/clear");	

	print "\n";
	print "==============================================================================\n";
	print "Usage : $0 \t[--installdir] [--type] [--external] [--external-type]\n"; 
	print "           \t\t[--bridge] [--blocklist] [--allowlist] [--setprefix]\n";
	print "           \t\t[--url] [--silent] [--userdefault] [--port]\n";
	print "           \t\t[--maxpolicysize]\n";
	print "==============================================================================\n";
	print "\n";

	exit;
}

#############################################################################
#loadCMDData : read and parse CMD supplied data                             #
#              installdir    - installation directory                       #
#              type          - deploy type (u/r/b)                          #
#              external      - exteranl interface                           #
#              external-type - external interface type                      #
#              bridge        - brifge id                                    #
#              blocklist     - blpck list                                   #
#              allowlist     - allow list                                   #
#              setprefix     - rules prefix (TS)                            #
#              dig           - path to dig command                          #
#              url           - url to upload logs                           #
#              silent        - suppress all output. Good for automatic      #
#                              run                                          #
#              port          - DNS port                                     #
#              maxpolicysize - MAX size of policies list                    #
#############################################################################
sub loadCMDData() {

	&printMsg("DEBUG","Parsing CMD supplied parameters.");

	GetOptions ($OPTS, 'installdir:s', 'type:s',
        		   'external:s', 'external-type:s', 'bridge:s', 'blocklist:s',
        		   'allowlist:s', 'setprefix:s', 'dig:s',
        		   'url:s', 
        		   'silent', 'usedefaults', 'help', 'port:i', 'maxpolicysize:i');

	#set source path to be absolute path to where setup script resides 
	$OPTS->{'src'} = $MYDIR;	

	#Check and set target install path to be absolute. Also copy all relevant
	#files into install dir
	if ( ! defined $OPTS->{'installdir'} ) {
                $OPTS->{'installdir'} = $OPTS->{'src'};
        } else {
		$OPTS->{'installdir_set'} = 1;
		if (substr($OPTS->{'installdir'},0,1) ne '/') {
			$OPTS->{'installdir'} = getcwd() . "/$OPTS->{'installdir'}";
		}
	}

	#Print usage message and exit
	if ( defined $OPTS->{help} ) {
		&usage();
		exit;
	}

	#Setting general parameters
	if ( ! defined $OPTS->{'setprefix'}) {
	        $OPTS->{'setprefix'} = $PREFIX;
	} else {
		$OPTS->{'setprefix_set'} = 1;
	}
	
	if ( ! defined $OPTS->{'blocklist'}) {
	        $OPTS->{'blocklist'} = $BLOCK_LIST;
	} else {
		$OPTS->{'blocklist_set'} = 1;
	}

	if ( ! defined $OPTS->{'allowlist'}) {
	        $OPTS->{'allowlist'} = $ALLOW_LIST;
	} else {
		$OPTS->{'allowlist_set'} = 1;
	}

	if ( ! defined $OPTS->{'url'}) {
	        $OPTS->{'url'} = $URL;
	} else {
		$OPTS->{'url_set'} = 1;
	}

	if ( ! defined $OPTS->{'dig'}) {
		$OPTS->{'dig'} = 'SET_ME';
	} else {
		$OPTS->{'dig_set'} = 1;
	}

	if ( ! defined $OPTS->{'logfile'}) {
	        $OPTS->{'logfile'} = $LOGFILE;
	} else {
		$OPTS->{'logfile_set'} = 1;
	}

	if ( ! defined $OPTS->{'type'} ) {
		$OPTS->{'type'} = $TYPE;
	} else {
		 $OPTS->{'type_set'} = 1;
		$OPTS->{'type'} = lc(substr($OPTS->{'type'},0,1));
	}

	if ( ! defined $OPTS->{'port'} ) {
		$OPTS->{'port'} = $PORT;
	} else {
		$OPTS->{'port_set'} = 1;
	}

	if ( ! defined $OPTS->{'maxpolicysize'} ) {
		$OPTS->{'maxpolicysize'} = $MAXPOLICYSIZE;
	} else {
		#We can not exceed max policy size supported by the system
		if ( $OPTS->{'maxpolicysize'} >  $MAXPOLICYALLOW ) {
			&printMsg("ERROR","The chosen policy max size [$OPTS->{'maxpolicysize'}] value exceeds the max policy size [$MAXPOLICYALLOW] supported by the system.");
			return 1;
		}
		#Everything is ok, use supplied value
		$OPTS->{'maxpolicysize_set'} = 1;
	}
	$OPTS->{'block_ipsetsize'} = $OPTS->{'maxpolicysize'};
	$OPTS->{'allow_ipsetsize'} = ( int( $OPTS->{'maxpolicysize'} * $ALLOW_IPSETPERCENTAGE / 100 ) ) < $ALLOW_MINIPSETSIZE ? $ALLOW_MINIPSETSIZE : int( $OPTS->{'maxpolicysize'} * $ALLOW_IPSETPERCENTAGE / 100 );

	#Setting additional parameters
	$OPTS->{'bridge'}                  = 'br0' if ( ! defined $OPTS->{'bridge'});

	#Intialise silent settings if needed
	if ( defined $OPTS->{'silent'}) {
		$DEBUG_LEVEL = 1;
		system("/usr/bin/clear");		
	}

	return 0;
}

##########################################################
#setCMDdataPRovidedPArams: set additional params based on#
#                           CMD supplied data            #
##########################################################
sub setCMDdataPRovidedPArams() {

	&printMsg("DEBUG", "Setting additional parameters based on CMD supplied data.");

        $OPTS->{'bridgeinrule'}            = $OPTS->{'setprefix'}.'bridgeinrule';
        $OPTS->{'bridgelocalrule'}         = $OPTS->{'setprefix'}.'bridgelocalrule';
        $OPTS->{'bridgeinrule'.$_}         = 10 + $_ for (0..7);
        $OPTS->{'bridgelocalrule'.$_}      = 10 + $_ for (0..7);
        $OPTS->{'bridgeinrule-default'}    = qq(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'default-action' 'accept');
        $OPTS->{'bridgelocalrule-default'} = qq(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'default-action' 'accept');

        #$OPTS->{'external'}            = 'eth0'     if ( ! defined $OPTS->{'external'});
        $OPTS->{'external-type'}       = 'ethernet' if ( ! defined $OPTS->{'external-type'});
        $OPTS->{'rtinrule'}            = $OPTS->{'setprefix'} . 'rtinrule';
        $OPTS->{'rtoutrule'}           = $OPTS->{'setprefix'} . 'rtoutrule';
        $OPTS->{'rtlocalrule'}         = $OPTS->{'setprefix'} . 'rtlocalrule';
        $OPTS->{'rtinrule'.$_}         = 10 + $_ for (0..3);
        $OPTS->{'rtoutrule'.$_}        = 10 + $_ for (0..3);
        $OPTS->{'rtlocalrule'.$_}      = 10 + $_ for (0..3);
        $OPTS->{'rtinrule-default'}    = qq(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'default-action' 'accept');
        $OPTS->{'rtlocalrule-default'} = qq(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'default-action' 'accept');
        $OPTS->{'rtoutrule-default'}   = qq(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'default-action' 'accept');

	return 0;
}

###############################################################
#loadPrevConfData : load data from configuration file created #
#                   during previous installation.             #
###############################################################
sub loadPrevConfData() {

	my $data_hash = {};
	my $err_msg;

	&printMsg("DEBUG","Verifying previous configuration existence.");

	# make sure we have at least the defaults file
	if ( defined "$CONFILE" and ! -e "$CONFILE") {

		&printMsg("INFO","Setting up default configuration");
		if ( ! copy $DEFAULTCONFILE, $CONFILE ) {

			&printMsg("ERROR", $!);
			return 1;
		}
	}

	if ( &loadConfFileData($CONFILE,$data_hash, \$err_msg) != 0 ) {

		&printMsg("ERROR", $err_msg);
		return 1;
	} 

	# Read data from conf file and load it into our OPTS hash overwriting the default values
	# Later user can exam the settings and adjust them manually if needed
	# Assumes that command line processing loaded some defaults
	$OPTS->{'allowlist'}     = $data_hash->{'allow_list'}    if ( ! defined $OPTS->{'allowlist_set'} && defined $data_hash->{'allow_list'} );
	$OPTS->{'blocklist'}     = $data_hash->{'block_list'}    if ( ! defined $OPTS->{'blocklist_set'} && defined $data_hash->{'block_list'} );
	$OPTS->{'url'}           = $data_hash->{'url'}           if ( ! defined $OPTS->{'url_set'}       && defined $data_hash->{'url'} );
	$OPTS->{'setprefix'}     = $data_hash->{'tsprefix'}      if ( ! defined $OPTS->{'setprefix_set'} && defined $data_hash->{'tsprefix'} );
	$OPTS->{'port'}          = $data_hash->{'port'}          if ( ! defined $OPTS->{'port_set'}      && defined $data_hash->{'port'} );
        if ( ! defined $OPTS->{'maxpolicysize_set'} && defined $data_hash->{'maxpolicysize'} ) {

		$OPTS->{'maxpolicysize'} = $data_hash->{'maxpolicysize'} <= $MAXPOLICYALLOW
			? $data_hash->{'maxpolicysize'}
			: $MAXPOLICYALLOW ;
	}

	$DNS_LIST = $data_hash->{'dns_server'} if ( defined $data_hash->{'dns_server'} );

	return 0;
}

##################################################################
# loadSRVData - 
# Give the services updater a chance to tweak config file settings.
# The config file may get re-written.
# The code is in that script because it will also get run from cron.
#
# returns :
# 0 - always...if we cannot get an SRV, we'll continue with what
#     was in the config file
sub loadSRVData {

	&printMsg("DEBUG","Calling '$UPDATE_SVCS_SCRIPT'");
	my $out = qx{$UPDATE_SVCS_SCRIPT};
	if ( $? != 0 ) {

		&printMsg("DEBUG", "script output : $out" );
        }

	return 0;
}

##################################################################
sub setInstallDir() {

	my $rc;
	my $rv;	

	&printMsg("INFO","Setting up install directory [$OPTS->{'installdir'}]");

	#If we are installing into differnt then SRC directory than make sure
	#it is exiss
	if ( ! -e $OPTS->{'installdir'} ) {
		&printMsg("DEBUG","Creating install directory [$OPTS->{'installdir'}]");
		$rc = qx(sudo mkdir -p $OPTS->{'installdir'} 2>&1);
		$rv = $?;

		if ( $rv != 0 ) {
			&printMsg("ERROR","Failed to create install directory [$OPTS->{'installdir'}]");
			&printMsg("DEBUG","Error - $rc");
			return 1;
		}
	}

	return 0;
}

###################################################################
#loadAdditionalSettings: Load additional settings extracted from
#                        system configuration
###################################################################
sub loadAdditionalSettings() {

	my ($platform, $platform_full);

	&printMsg("DEBUG","Loading additional required system settings.");
	
	#Load apliance version, 0 if could not detect
	&ipSetVerDetector();
	&printMsg("DEBUG","Detecting current ipset version - [$OPTS->{'ipsetver'}]");

	&OSVerDetector();
	&printMsg("DEBUG","Detecting OS version - [$OPTS->{'osverfull'}]");

	#Load system bits data 64/32
	&osPlatformDetecter(\$platform,\$platform_full);
	$OPTS->{'platform'} = $platform;
	$OPTS->{'platform_full'} = $platform_full;
	&printMsg("DEBUG","Detecting system platform - [$OPTS->{'platform_full'}]");

	# need to check for some additional configurations
	chomp (my $dig = qx(sudo which dig)); 
	if ( ! -x $dig ) {

		&printMsg("DEBUG","loadAdditionalSettings() : dig NOT found by 'sudo which dig'");
		&printMsg("ERROR","The prerequisite \"dig\" utility was not detected in root's path;");
		&printMsg("ERROR","... try installing the \"dnsutils\" package.");
		&printMsg("ERROR","(refer to https://help.ubnt.com/hc/en-us/articles/205202560-EdgeMAX-Add-other-Debian-packages-to-EdgeOS)");
		return 1;
	}
	&printMsg("DEBUG","loadAdditionalSettings() : dig found at '$dig'");
	$OPTS->{'dig'} = $dig;

	chomp (my $lwp = qx(sudo perl -MLWP::UserAgent -e 'print "FOUNDLWP";' 2>&1)); 
	if ( $lwp !~ /^FOUNDLWP$/ ) {

		&printMsg("DEBUG","loadAdditionalSettings() : LWP::UserAgent NOT found in perl's library search path'");
		&printMsg("ERROR","The prerequisite perl module LWP::UserAgent was not found;");
		&printMsg("ERROR","... try installing the \"libwww-perl\" package.");
		&printMsg("ERROR","(refer to https://help.ubnt.com/hc/en-us/articles/205202560-EdgeMAX-Add-other-Debian-packages-to-EdgeOS)");
		return 1;
	}
	&printMsg("DEBUG","loadAdditionalSettings() : LWP::UserAgent found");

	#Depending on OS version we need to update a few settings
	if ( ! defined $OPTS->{'logfile_set'} ) {
		if ( $OPTS->{'osver'} >= $OS_VER_TRIGGER ) {
			$OPTS->{'logfile'} = $LOGALTFILE;
			$OPTS->{'logsys'}  = $LOGRSYSLOG;
		}
	}

	return 0;
}

####################################################################
#askUserDo : load interactively supplied settings                  #
####################################################################
sub askUserDo() {
	my $opt;

	#If we are running in silent mode no need to prompt for user input
	return 0 if ( defined $OPTS->{'silent'} );

	#Set target installation dir. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'installdir_set'} ) {
		print "\nPlease enter the Threatstop installation directory: [$OPTS->{'installdir'}] ";
		$opt = <STDIN>;
		chomp($opt);
	
		if ( $opt ne "" ) {
			if ( -d $opt ) { $OPTS->{'installdir'} = $opt; }
			else {
				print "[ATTENTION] : Invalid Threatstop installation directory [$opt] provided.\nRestarting...\n";
				return 1;
			}
		}
	}

	#Set installation prefix. Do not prompt if set from CLI 
	if ( ! defined $OPTS->{'setprefix_set'} ) {
		print "\nPlease enter the Threatstop ipset prefix: [$OPTS->{'setprefix'}] ";
		$opt = <STDIN>;
		chomp($opt);
	        $OPTS->{'setprefix'} = $opt if ( $opt ne "");
	}

	#Set installation type. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'type_set'} ) {
		$OPTS->{'type'} = uc $OPTS->{'type'};
		print "\nPlease enter the configuration type ((B)ridge/(R)outer/(U)ntouched): [$OPTS->{'type'}] ";
		$opt = <STDIN>;
		chomp($opt);

		#if type is not u/r/b than re-ask. Do not prompt if set from CLI
		switch ( $opt ) {
			case /^[b|B]$/ { ;; }
			case /^[r|R]$/ { ;; }
			case /^[u|U]$/ { ;; }
			case /^$/      { ;; } 
			else {
				print "[ATTENTION] : Bad option [$opt] chosen.\nRestarting...\n";
	                        return 1;
			}
		}

		$OPTS->{'type'} = $opt if ( $opt ne "");
		$OPTS->{'type'} = lc(substr($OPTS->{'type'},0,1));
	}

	#Set bridge settings. Do not prompt if set from CLI	
	if ( $OPTS->{'type'} eq 'b' ) {
		if ( ! defined $OPTS->{'bridge_set'} ) {
			print "\nPlease enter the bridge id (e.g. br0): [$OPTS->{'bridge'}] ";
			$opt = <STDIN>;
			chomp($opt);

			$OPTS->{'bridge'} = $opt if ( $opt ne "");
		}

		rulenamestart ($OPTS->{'bridge'}, 'in');
               	rulenamestart ($OPTS->{'bridge'}, 'local');	
	}

	#Set router settings	
	if ( $OPTS->{'type'} eq 'r' ) {

		if ( ! defined $OPTS->{'external_set'} ) {		

			if (  ! defined $OPTS->{'external'} ) {
				#Set default value
				$OPTS->{'external'} = 'eth0';

				#Queiry defined interface list. Do not prompt if set from CLI
				print "\n";
				my $cmd = qq(exec vbash -i -c "show interfaces");
				my $rc  = qx($cmd);

				print "Available interfaces\n";
				print "$rc\n";
				print "External interface id: [$OPTS->{'external'}] ";

				#Ask user to choose
				$opt = <STDIN>;
				chomp($opt);
				$OPTS->{'external'} = $opt if ( $opt ne "");	
			} else {
				print "Install type routed - external interface [$OPTS->{'external'}]\n";
			}
		}

		if ( ! defined $OPTS->{'external-type_set'} ) { 
			$OPTS->{'external-type'} = 'ethernet';

			if ( $OPTS->{'external'} !~ /eth\d/ ) {
	
				print "External interface type (ethernet, wireless, adsl, multilink): [$OPTS->{'external-type'}] ";
				$opt = <STDIN>;
				chomp($opt);
				$OPTS->{'external-type'} = $opt if ( $opt ne "");

				#Verify user input... check for typof
				switch ( $OPTS->{'external-type'} ) {
					case 'ethernet'  { ;; }
					case 'wireless'  { ;; }
					case 'adsl'      { ;; }
					case 'multilink' { ;; }
					else { $OPTS->{'external-type'} = "Unrecognized"; }
				}
			}
		} 
		rulenamestart ($OPTS->{'external'}, 'in');
                rulenamestart ($OPTS->{'external'}, 'local');
                rulenamestart ($OPTS->{'external'}, 'out');
	}
 
	#set block list setting. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'blocklist_set'} ) {
		print "\nPlease enter the Threatstop block list parameter: [$OPTS->{'blocklist'}] ";
		$opt = <STDIN>;
		chomp($opt);
		$OPTS->{'blocklist'} = $opt if ( $opt ne "");
	}

	#set allow list setting. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'allowlist_set'} ) {
		print "\nPlease enter the Threatstop allow list parameter: [$OPTS->{'allowlist'}] ";
		$opt = <STDIN>;
		chomp($opt);
		$OPTS->{'allowlist'} = $opt if ( $opt ne "");
	}

	#set dig command location. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'dig_set'} ) {
		print "\nPlease enter the location of the dig command: [$OPTS->{'dig'}] ";
		$opt = <STDIN>;
		chomp($opt);
		if ( $opt ne "" ) {
			if ( -e $opt ) { $OPTS->{'dig'} = $opt; }
			else {
				print "[ATTENTION] : Invalid location of the dig command [$opt] provided.\nRestarting...\n";
				return 1;
			}
		}
	}

	#set logfile setting. Do not prompt if set from CLI
	# GS-351 : commenting all out, since the <STDIN> was previously commented by someone
	#if ( ! defined $OPTS->{'logfile_set'} ) {
	#	print "\nPlease enter the Threatstop logfile location: [$OPTS->{'logfile'}] ";
	#	$opt = <STDIN>;
	#	$opt = "";
	#	chomp($opt);
	#	$OPTS->{'logfile'} = $opt if ( $opt ne "");
	#}

	#set URL setting. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'url_set'} ) {
		print "\nPlease enter the URL parameter for submitting logs: [$OPTS->{'url'}] ";
		$opt = <STDIN>;
		chomp($opt);
		$OPTS->{'url'} = $opt if ( $opt ne "");
	}

	#set DNS port setting. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'port_set'} ) {
		print "\nPlease enter the DNS PORT setting: [$OPTS->{'port'}] ";
		$opt = <STDIN>;
		chomp($opt);
		if ( $opt ne "" ) {
			if ( $opt =~ /^\s*(\d+)\s*$/ && $1 > 0 ) { $OPTS->{'port'} = $1; }
			else {
				print "[ATTENTION] : Invalid DNS PORT setting [$opt] provided.\nRestarting...\n";
				return 1;
			}
		}
	}	

	#set MAXPOLICYSIZE setting. Do not prompt if set from CLI
	if ( ! defined $OPTS->{'maxpolicysize_set'} ) {
                my $default = defined $OPTS->{'maxpolicysize'} && $OPTS->{'maxpolicysize'} <= $MAXPOLICYALLOW ? $OPTS->{'maxpolicysize'} : $MAXPOLICYALLOW;
		print "\nPlease provide a MAX POLICY SIZE value which is less than or equal to $MAXPOLICYALLOW: [$default] ";
        	$opt = <STDIN>;
        	chomp($opt);
		if ( $opt ne "" ) {
			if ( $opt =~ /^\s*(\d+)\s*$/ && $1 > 0 ) { $OPTS->{'maxpolicysize'} = $1; }
			else {
				$OPTS->{'maxpolicysize'} = $default;
				print "[ATTENTION] : Invalid MAX POLICY SIZE value [$opt] provided.\nRetaining the original value [$default] and restarting...\n";
				return 1;
			}
		} else {
			$OPTS->{'maxpolicysize'} = $default;
		}
		$OPTS->{'block_ipsetsize'} = $OPTS->{'maxpolicysize'};
		$OPTS->{'allow_ipsetsize'} = ( int( $OPTS->{'maxpolicysize'} * $ALLOW_IPSETPERCENTAGE / 100 ) ) < $ALLOW_MINIPSETSIZE ? $ALLOW_MINIPSETSIZE : int( $OPTS->{'maxpolicysize'} * $ALLOW_IPSETPERCENTAGE / 100 );
	}

	return 0;
}

######################################################################
#verifyUploadedSettings : prints settings including user supplied    #
#                         before proceed with installation           #
######################################################################
sub verifyUploadedSettings() {
	
	my $isOK;
	my $opt;

	return 0 if ( $OPTS->{'silent'} );

	&printMsg("INFO","Entering user interactive setup stage.");

	while (1) {

		$isOK = 1;

		next if ( &askUserDo() != 0 );

		#See if we exceed MAX SYSTEM ALLOW number
		if ( $OPTS->{'maxpolicysize'} > $MAXPOLICYALLOW ) {
			&printMsg("ERROR","The provided MAXPOLICYSIZE value [$OPTS->{'maxpolicysize'}] exceeds maximum policy size allowed by system [$MAXPOLICYALLOW]");
			next; 
		}

		&printMsg("DISPL","\n");
		&printMsg("DISPL", "Chosen configuration parameters");
		&printMsg("DISPL", "--------------------------------------------------------");
		&printMsg("DISPL", "Installation directory : $OPTS->{'installdir'}");
		&printMsg("DISPL", "Installation prefix    : $OPTS->{'setprefix'}");
		if ( $OPTS->{'type'} eq "b" ) {
			&printMsg("DISPL", "Installation type      : Bridge");
			&printMsg("DISPL", "Bridge ID              : $OPTS->{'bridge'}");
		} elsif ( $OPTS->{'type'} eq "r" )  {
			&printMsg("DISPL", "Installation type      : Router");	
			&printMsg("DISPL", "External interface type: $OPTS->{'external-type'}");
			$isOK = 0 if ( $OPTS->{'external-type'} eq "Unrecognized" );
		} elsif ( $OPTS->{'type'} eq "u" ) {
			&printMsg("DISPL", "Installation type      : Unchanged");
		} else {
			&printMsg("DISPL", "Installation type      : Unrecognized");
			$isOK = 0;
		}
		&printMsg("DISPL", "Threatstop Block list  : $OPTS->{'blocklist'}");
		if ( $OPTS->{'allowlist'} eq '' ) {
			&printMsg("DISPL", "Threatstop Allow list  : Unrecognized");
			$isOK = 0;
		} else {
			&printMsg("DISPL","Threatstop Allow list  : $OPTS->{'allowlist'}");
		}
		&printMsg("DISPL", "Dig command location   : $OPTS->{'dig'}");
		&printMsg("DISPL", "Logfile location       : $OPTS->{'logfile'}");
		&printMsg("DISPL", "Upload logs URL        : $OPTS->{'url'}");
		&printMsg("DISPL", "DNS port               : $OPTS->{'port'}");
		&printMsg("DISPL", "Max policy size        : $OPTS->{'maxpolicysize'}");
		&printMsg("DISPL", "Allow ipset size       : $OPTS->{'allow_ipsetsize'}");
		&printMsg("DISPL", "Block ipset size       : $OPTS->{'block_ipsetsize'}");
		&printMsg("DEBUG", "DNS server             : $DNS_LIST");
		&printMsg("DISPL", "--------------------------------------------------------");
		print "Use these settings (Y/N)?: [Y] ";
		$opt=<STDIN>;
		chomp($opt);

		next if ( $opt =~ /[nN]/ );
		last if ( $isOK ==  1 );
		print "\n******************************************************************\n";
		print "WARNING : Some of mandatory parameters had not been properly set...\n";
		print "*******************************************************************\n";
	}

	print "\n";

	return 0;
}

######################################################################
# determineHashType - as of EdgeOS 1.5.0, it's become necessary to
# extract the actual hash type that the ipset for address-group
# is created with.  In the past, it's always been :
#     group type       |  hash type
#     =============================
#     address-group    |  hash:ip
#     network-group    |  hash:net
#
# calls external helper script : extractHashType.sh
#
# globals affected :
# $OPT->{'ipset_hash_type'}
#
# returns
# 0 for success
# 1 for failure
#
# refer to GS-674
######################################################################
sub determineHashType {

    $OPTS->{'ipset_hash_type'} = undef;

    my $cmd = "sg vyattacfg $OPTS->{'src'}/extractHashType.sh";
    chomp(my $output = qx($cmd 2>&1));

    if ( $? != 0 ) {

        &printMsg("ERROR","Could not extract the hash type for an ipset created for an address-group");
        return 1;
    }

    if ( $output ne 'hash:ip' and $output ne 'hash:net' ) {

        &printMsg("ERROR","Probed invalid ipset type for an address-group on this system : $output");
        return 1;
    }

    # set globally, for the config file, which gets save soon
    $OPTS->{'ipset_hash_type'} = $output;
    &printMsg("DEBUG","Detected ipset hash type for address-group : $OPTS->{'ipset_hash_type'}");
   
    return 0;
}

######################################################################
#generateConfFile : write seetings into config file for future usage #
######################################################################
sub generateConfFile() {

	&printMsg("INFO", "Creating config file [$OPTS->{'src'}/threatstop.conf]");

	eval {
		open (FILE, '>' . $OPTS->{'src'} . '/threatstop.conf') or die "Opening config file failed with error [$!]";
	};

	if ( $@ ) {
		&printMsg("ERROR","$@");
		return 1;
	}
	my $t = qx(date);
	print FILE "# Threatstop.conf for vyatta created by setup.pl v$VERSION at $t\n";
	print FILE "\n";
	print FILE "# Dig command path\n";
	print FILE "dig=\"$OPTS->{'dig'}\"\n";
	print FILE "\n";
	print FILE "# Full path to the ipset executable\n";
	print FILE "ipset=\"$OPTS->{'ipset'}\"\n";
	print FILE "# version of the ipset executable\n";
	print FILE "ipsetver=\"$OPTS->{'ipsetver'}\"\n";
	print FILE "\n";
	print FILE "# Ipset prefix for threatstop sets\n";
	print FILE "tsprefix=\"$OPTS->{'setprefix'}\"\n";
	print FILE "\n";
	print FILE "# ThreatSTOP DNS Servers\n";
	print FILE "dns_server=\"$DNS_LIST\"\n";
	print FILE "\n";
	print FILE "# The block and/or allow list\n";
	print FILE "block_list=\"$OPTS->{'blocklist'}\"\n";
	print FILE "allow_list=\"$OPTS->{'allowlist'}\"\n";
	print FILE "\n";
	print FILE "#logfile name\n";
	print FILE "logfile=\"$OPTS->{'logfile'}\"\n";
	print FILE "\n";
	print FILE "#optional IP to upload the log files as\n";
	print FILE "#uncomment and set to an ip that you have registered at threatstop.com if\n";
	print FILE "#your real IP address is different than what you have set up at threatstop.com\n";
	print FILE "#loguploadip=127.0.0.1\n";
	print FILE "\n";
	print FILE "#url to upload logfiles to\n";
	print FILE "url=\"$OPTS->{'url'}\"\n";
	print FILE "\n";
	print FILE "#Port to use for DNS queries\n";
	print FILE "port=\"$OPTS->{'port'}\"\n";
	print FILE "\n";
	print FILE "#ipset sizes\n";
	print FILE "allow_ipsetsize=\"$OPTS->{'allow_ipsetsize'}\"\n";
	print FILE "block_ipsetsize=\"$OPTS->{'block_ipsetsize'}\"\n";
	print FILE "\n";
	print FILE "#Maximum size of policies list\n";
	print FILE "maxpolicysize=\"$OPTS->{'maxpolicysize'}\"\n";
	print FILE "\n";
	close FILE;
	
	return 0;
}

####################################################
#testingConnection : test access to DNS servers    #
#      $DNS_LIST is default variable loaded from   #
#                lib/ThreatStop.pm module          #
####################################################
sub testingConnection() {

	my @dns_ok_list;

	#Testing DNS servers access. Also verify dig command
	#loaded from lib/Threatstop.pm module
	if ( &checkDNSaccess($OPTS->{'dig'},$DNS_LIST,\@dns_ok_list, $OPTS->{'port'}) != 0 ) {
		return 1;
	} 

	&printMsg("DEBUG","Using DNS servers list [@dns_ok_list]");
	return 0;
}

###################################################################################
#generateRouteRuleSet : generates rules list and stores them in configure.route.sh#
#                       shell file which will be run later                        #
###################################################################################
sub generateRouteRuleSet() {

	&printMsg("DEBUG","Generating route rule set");
	my $nameserver_ips_ref = shift;

	$OPTS->{'merge'}=$OPTS->{'installdir'}.'/configure.route.sh';

	eval {
		open (FILE, '>'.$OPTS->{'merge'}) or die "Opening route rule set file failed with error [$!]";
	};

	if ( $@ ) {
		&printMsg("ERROR","$@");
		return 1;
	}

	print FILE <<ROUTEFW;
#!/usr/bin/env bash
set -x
#make sure we are group vyattacfg this may cause us to exec ourselves with sg
#setGID must be called with $@ as an argument or it will not preserve the 
#command line arguments.
function setGID() {
        local GRP=\$(id -gn)
        local uid=\$(id -u )
        if [ "\$uid" == "0" ]; then
            echo "Refusing to run as root."
            exit 1
        fi
        if [ "\$GRP" != "vyattacfg" ]; then
                echo "Setting our group to vyattacfg."
                umask 002
                exec sg vyattacfg "\$0 \$*"
        fi
}
setGID \$@
if [ "\$1" != "revert" ]; then
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}allownet' 'network' '24.249.204.0/29'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet' 'network' '169.254.0.0/16'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'log-martians' 'enable'
$OPTS->{'rtinrule-default'}
$OPTS->{'rtlocalrule-default'}
$OPTS->{'rtoutrule-default'}
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'rule' '$OPTS->{'rtinrule1'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'rule' '$OPTS->{'rtinrule1'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'rule' '$OPTS->{'rtinrule3'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'rule' '$OPTS->{'rtinrule3'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtinrule'}' 'rule' '$OPTS->{'rtinrule3'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'rule' '$OPTS->{'rtlocalrule1'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'rule' '$OPTS->{'rtlocalrule1'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'rule' '$OPTS->{'rtlocalrule3'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'rule' '$OPTS->{'rtlocalrule3'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtlocalrule'}' 'rule' '$OPTS->{'rtlocalrule3'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'rule' '$OPTS->{'rtoutrule1'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'rule' '$OPTS->{'rtoutrule1'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'rule' '$OPTS->{'rtoutrule3'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'rule' '$OPTS->{'rtoutrule3'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'rtoutrule'}' 'rule' '$OPTS->{'rtoutrule3'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'syn-cookies' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'out' 'name' '$OPTS->{'rtoutrule'}'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'in' 'name' '$OPTS->{'rtinrule'}'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'local' 'name' '$OPTS->{'rtlocalrule'}'
else
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'out' 'name' '$OPTS->{'setprefix'}rtoutrule'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'in' 'name' '$OPTS->{'setprefix'}rtinrule'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'interfaces' '$OPTS->{'external-type'}' '$OPTS->{'external'}' 'firewall' 'local' 'name' '$OPTS->{'setprefix'}rtlocalrule'
# GS-674 : leave this next "commit" in; at least EdgeOS needs the
# previous three to be committed, before the next three will succeed
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'name' '$OPTS->{'setprefix'}rtinrule' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'name' '$OPTS->{'setprefix'}rtlocalrule' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'name' '$OPTS->{'setprefix'}rtoutrule' 
#/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}allownet' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet' 
fi
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper save
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper end
ROUTEFW
        close FILE;
	print qx(chmod a+x $OPTS->{'merge'});
	return 0;
}

########################i##########################################################
#generateBridgeRuleSet: generates rules list and stores them in configure.route.sh#
#                       shell file which will be run later                        #
###################################################################################
sub generateBridgeRuleSet() {

	&printMsg("DEBUG","Generating bridge rule set.");
	my $nameserver_ips_ref = shift;

	$OPTS->{'merge'}=$OPTS->{'installdir'}.'/configure.bridge.sh';

        eval {
                open (FILE, '>'.$OPTS->{'merge'}) or die "Opening route rule set file failed with error [$!]";
        };

        if ( $@ ) {
                &printMsg("ERROR","$@");
                return 1;
        }

	print FILE <<BRIDGEFW;
#!/usr/bin/env bash
set -x
#make sure we are group vyattacfg this may cause us to exec ourselves with sg
#setGID must be called with $@ as an argument or it will not preserve the 
#command line arguments.
function setGID() {
        local GRP=\$(id -gn)
        local uid=\$(id -u )
        if [ "\$uid" == "0" ]; then
            echo "Refusing to run as root."
            exit 1
        fi
        if [ "\$GRP" != "vyattacfg" ]; then
                echo "Setting our group to vyattacfg."
                umask 002
                exec sg vyattacfg "\$0 \$*"
        fi
}
setGID \$@
if [ "\$1" != "revert" ]; then
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}allownet' 'network' '24.249.204.0/29'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet' 'network' '169.254.0.0/16'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'log-martians' 'enable'
$OPTS->{'bridgeinrule-default'}
$OPTS->{'bridgelocalrule-default'}
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule1'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule1'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule3'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule3'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule5'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule5'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule5'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule7'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule7'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 'rule' '$OPTS->{'bridgeinrule7'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule1'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule1'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule3'}' 'action' 'accept'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule3'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}allownet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule5'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule5'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule5'}' 'source' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule7'}' 'action' 'drop'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule7'}' 'destination' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 'rule' '$OPTS->{'bridgelocalrule7'}' 'log' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'firewall' 'syn-cookies' 'enable'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'interfaces' 'bridge' '$OPTS->{'bridge'}' 'firewall' 'in' 'name' '$OPTS->{'bridgeinrule'}'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set 'interfaces' 'bridge' '$OPTS->{'bridge'}' 'firewall' 'local' 'name' '$OPTS->{'bridgelocalrule'}'
else
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'interfaces' 'bridge' '$OPTS->{'bridge'}' 'firewall' 'in' 'name' '$OPTS->{'bridgeinrule'}'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'interfaces' 'bridge' '$OPTS->{'bridge'}' 'firewall' 'local' 'name' '$OPTS->{'bridgelocalrule'}'
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'name' '$OPTS->{'bridgeinrule'}' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'name' '$OPTS->{'bridgelocalrule'}' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}allownet' 
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper delete 'firewall' 'group' 'network-group' '$OPTS->{'setprefix'}blocknet' 
fi
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper save
/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper end
BRIDGEFW

	close FILE;
        print qx(chmod a+x $OPTS->{'merge'});
	return 0;
}

############################################################################
#generateRules: decides what rules set will be created based on deploy type#
############################################################################
sub generateRules() {

	&printMsg("DEBUG","Ready to generate rule sets.");

	# both the following will need to set allow rules based upon the TS name servers
	# NOTE : a recent refactor makes $DNS_LIST a single item.  It will typically
	# start out as an anycast IP address.  SRV lookup will likely quickly take it
	# to a single hostname of the name server we should prefer.  In this latter case
	# we'll grab all A records for this host, and pass them on for setting with
	# 'allow' rules.
	my @DNS_LIST_IPS = ();
        if ( $DNS_LIST =~ /^\d+\.\d+\.\d+\.\d+$/ ) {

		&printMsg("DEBUG", "DNS server list is an IP");
		push @DNS_LIST_IPS, $DNS_LIST;
	} else {

		if ( ! $OPTS->{'dig'} ) {

			&printMsg("DEBUG", "couldn't find the dig command");
			return 1;
		}
        
		&printMsg("DEBUG", "looking up A records for $DNS_LIST");
		my $ips = `$OPTS->{'dig'} +tcp +short -t a $DNS_LIST 2>&1`;

		my $status = $?;
  
		&printMsg("DEBUG", "A query status ($status) return : $ips");
		if ( $status != 0 ) {

			&printMsg("DEBUG", "dig failed");
			return 1;
		}

		foreach my $ip ( split(/\n/, $ips) ) {

			&printMsg("DEBUG", "working on line : '$ip'");

			if ( $ip =~ /^\s+$/ ) {

				&printMsg("DEBUG", "Got an empty A '$ip'; skipping");
				next;
			}

		        if ( $ip !~ /^\d+\.\d+\.\d+\.\d+$/ ) {
 
				&printMsg("DEBUG", "Unexpected format for A record '$ip'");
				next;
			}

			push @DNS_LIST_IPS, $ip;
		}
	}
	if ( ! scalar @DNS_LIST_IPS ) {

		&printMsg("ERROR", "Unable to determine the ThreatStop name server IP(s)");
		return 1;
	}

	if ( $OPTS->{'type'} eq "r" ) {
		return 1 if ( &generateRouteRuleSet(\@DNS_LIST_IPS) != 0 );
		return 0;
	}

	if ( $OPTS->{'type'} eq "b" ) {
		return 1 if ( &generateBridgeRuleSet(\@DNS_LIST_IPS) != 0 );
		return 0;
	}

	&printMsg("DEBUG","Keeping current rule set unchanged.");

	return 0;
}

####################################################
#restartingSysLog : restarts syslog service        #
####################################################
sub restartingSysLog() {

	my $rc;
	my $rv;

	if ( $OPTS->{'osver'} < $OS_VER_TRIGGER_OLDER ) {
        &printMsg("INFO","Restarting syslog service.");
		$rc = qx(/etc/init.d/sysklogd reload);
                $rv = $?;
	} elsif ( $OPTS->{'osver'} < $OS_VER_TRIGGER ) {
        &printMsg("INFO","Restarting syslog service.");
		$rc = qx(service rsyslog reload);
               	$rv = $?;
	} else {
        &printMsg("INFO","Restarting rsyslog service.");
        $rc = qx(service rsyslog restart);
        $rv = $?;
	}

    if ( $rv != 0 ) {
    	&printMsg("ERROR","Restarting rsyslog service failed with error [$!]");
        return 1;
    }
	
	return 0;
}

sub deployLogMonitoringVyatta {
    #First remove any old syslog/logrotate stuff so we can cleanly upgrade from a previous version.

    #check if we already have threatstop.log set up in the system config before we try to set it up.
    qx(vbash -ic 'show configuration' | grep threatstop.log >/dev/null 2>&1);
    my $rv = $?;
    if ( $rv == 0 ) {
        &printMsg("DEBUG","Looks like we already have /var/log/user/threatstop.log set up. Doing nothing.");
        return 0;
    } else {
        #First remove any old syslog/logrotate stuff so we can cleanly upgrade from a previous version.

        #old school syslog
        #Digging through the code even in old versions we never touched /etc/syslog.conf and just used /var/log/messages instead.
        #so nothing to do in this case.

        #rsyslog
        if (-f "/etc/rsyslog.d/01-threatstop-rsyslog.conf") {
            unlink("/etc/rsyslog.d/01-threatstop-rsyslog.conf");
        }

        #remove our legacy logrotate.d file if it exists.
        if ( -f "/etc/logrotate.d/threatstop-logrotate.conf" ) {
            unlink("/etc/logrotate.d/threatstop-logrotate.conf")
        }

        #remove even older logrotate.d file if it exists.
        #XXX! The old script edited the logrotate file that vyatta put in for their own definition of /var/log/messages
        #we need to instead remove our logrotate from this script instead of attempting to remove it.
        if( -f "/etc/logrotate.d/messages") {
            $rv = system('grep -i logupload /etc/logrotate.d/messages');
            if($rv == 0) {
                &printMsg("DEBUG","We found our ancient logupload line in /etc/logrotate.d/messages. Removing it.");
                qx(sudo sed -i '/logupload/d' /etc/logrotate.d/messages);
            }
        }
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set system syslog file threatstop.log archive size 100);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set system syslog file threatstop.log facility kern level debug);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper set system syslog file threatstop.log facility syslog level debug);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper commit);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper save);
        print qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper end);
    }
}

###########################################################################
#generateCronJob : creates a crontab tmp file holding ThreatSTOP relevant
#                  entries. This file will be used later for uploading
#                  new jobs
# NOTE : this accounts for new vyatta-based versions that use rsyslog
# $outchannel support to fire logrotation when log files reach a designated
# size
###########################################################################
sub generateCronJob() {

	my $rotatefile;

	if ( -f "/etc/logrotate.d/threatstop.log" ) {
		$rotatefile = "/etc/logrotate.d/threatstop.log";
	} elsif ( -f "/opt/vyatta/etc/logrotate/file_threatstop.log" ) {
		$rotatefile = "/opt/vyatta/etc/logrotate/file_threatstop.log";
	} else {
		&printMsg("ERROR","Could not determine logrotate file path!");
		return 1;
	}

	&printMsg("DEBUG","Creating crontab job configuration.");

        # retain existing cron content
        system("sudo crontab -l >$OPTS->{'installdir'}/crontab 2>/dev/null;");

        # early-out, if our content appears to be there (based upon presence of
        # "ThreatSTOP" string in existing content)
        system("grep -q ThreatSTOP $OPTS->{'installdir'}/crontab");
        if ( $? == 0 ) {

		&printMsg("INFO","ThreatSTOP cron content found to already exist");
		return 0;
        }

        # append our content (revert.sh will selectively remove our content)
	eval {
		open (FILE, '>>'.$OPTS->{'installdir'}.'/crontab') or die "$!";
	};

	if ( $@ ) {
		&printMsg("ERROR","Creating local crontab failed with error [$!]");
		return 1;
	}

	my ($m, $h)=(gmtime())[1,2];
	$h = ($m+5)%60; # 5 minutes after ipsetget

        print FILE <<CRONTAB;
# Update the ThreatSTOP lists every 2 hours, $m minutes after the hour 
# (00:$m, 02:$m, 04:$m, etc.)
$m     */2     *     *     *     $OPTS->{'src'}/ipsetget.pl
CRONTAB
	# GS-714 :
	# if logrotate fires against the main config file, it will also process the
	# the threatstop config that's included in logrotate.d/
	# and all distributions appear to fire logrotate out of cron.hourly/* without
	# us having to do anything.  but just to be safe, we'll check.
	# Note, it also fires out of cron.daily, but that's not frequent enough for us.
	my $foundInRsyslogd = 0;
	`grep /var/log/user/threatstop.log /etc/rsyslog.d/* | grep -q -i logrotate`;
	if ( $? == 0 ) { $foundInRsyslogd = 1; }
	my $foundInCronHourly = 0;
	`grep -q logrotate /etc/cron.hourly/*`;
	if ( $? == 0 ) { $foundInCronHourly = 1; }

	if ( ! $foundInRsyslogd && ! $foundInCronHourly ) {
		&printMsg("DEBUG","Determined that we need to do logrotation ourselves.  Inserting logrotate cron job.");
		print FILE<<CRONTAB;
# Force a logrotate if the log is > 100k. Check every $h minutes after the hour
$h    *     *     *     *     /usr/sbin/logrotate -f $rotatefile   
CRONTAB
	} else {
		&printMsg("DEBUG","No need to add cron for logrotate; handled by rsyslog ($foundInRsyslogd) or cron hourly ($foundInCronHourly).");
	}

	# always need the service update script
	my $svc_min = ($m+30)%60; # 30mins offset from policy updater
	print FILE <<CRONTAB;
# Update services host/port
$svc_min     */2     *     *     *     $OPTS->{'src'}/update_services.pl
CRONTAB
	close FILE;

	return 0;
}

################################################################################
#generateRClocal : generates a rc.locla tmp file holding a threatSTOP relevant #
#                  entry                                                       #
################################################################################
sub generateRClocal() {

	&printMsg("DEBUG","Creating local copy of rc.local file.");
	
	#Read original rc.local file
	open (FILE, '</etc/rc.local');
	$_=join('',<FILE>);
	$ORGRCLOCAL= $_;
	close FILE;

	#Update ipsetget setting
	if (/\S+ipsetget.pl/) {
        	s/\S+ipsetget.pl/$OPTS->{'src'}\/ipsetget.pl/s;
	} else {
        	s/^exit 0/$OPTS->{'src'}\/ipsetget.pl\nexit 0/sm;
	}

	if ($_ eq $ORGRCLOCAL) {
        	&printMsg("DEBUG","Skipping, as rc.local already contains the correct command\n");
		$ORGRCLOCAL = 1;
	} else {
		eval {
	        	open (FILE, '>'.$OPTS->{'installdir'}.'/rc.local') or die "$!";
		};

		if ( $@ ) {
			&printMsg("ERROR","Opening local rc.local failed with error [$@]");
			return 1;
		}
        	print FILE $_;
        	close FILE;

		eval {
	        	open (FILE, '>'.$OPTS->{'installdir'}.'/rc.local.org') or die "$!";
		};

		if ( $@ ) {
			&printMsg("ERROR","Opening local rc.local.org failed with error [$@]");
                        return 1;
		}

        	print FILE $ORGRCLOCAL;
        	close FILE;

        	$ORGRCLOCAL="";
	}

	return 0;
}

######################################################################################
#deployMergeSettings: executes the previously created configure.route.sh script thus #
#                     effectively deploying created rules sets                       # 
######################################################################################
sub deployMergeSettings() {

	my $rc;
	my $rv;
	my $cmd;

	&printMsg("NOTE","Merging $OPTS->{'merge'}");

	$rv = qx(sg vyattacfg $OPTS->{'merge'} 2>&1);
	$rc=$?;

	if ( $rc != 0 ) {
		&printMsg("ERROR","vyattacfg command failed with error [$rv]");
		return 1;
	}

	#Print merge operation output 
	if ( $DEBUG_LEVEL > 5) {
		&printMsg("DEBUG","$OPTS->{'merge'} output.");
		print "\n$rv\n\n";
	}

	#See if we already have TS.org file in place. We do not want to overwrite it 
	&printMsg("DEBUG","Creating backup copy of current rules configuration.");
	if ( -e "$OPTS->{'installdir'}/TS.org" ) {
		&printMsg("DEBUG","Found previously stored rules configuration... Skipping TS.org backup file creation.");
		return 0;
	}

	#We are running for the first time. Create TS.org rules backup file	
	eval {
		open (FILE, '>'.$OPTS->{'installdir'}.'/TS.org') or die "$!";
	};	

	if ( $@ ) {
		&printMsg("ERROR","Opening TS.org file failed with error [$!]");
		return 1;
	}

	$cmd  = "sudo " if ( $UID != 0 );
	$cmd .= $OPTS->{'ipset'}.' -S';
	$cmd .= '| sort -r' if ( $OPTS->{'ipsetver'} =~ /v6.0/ ); #fix for ipset 6.0

        $rv  = qx($cmd 2>&1);
	$rc  = $?;

	if ( $rc != 0 ) {
		&printMsg("ERROR","ipset command failed with error [$rv]");
		return 1;
	}

        $rv =~ s/((?:allow|block)(?:net|addr))/$1new/sg;
        print FILE $rv;
        close FILE;

	return 0;
}

####################################################################################
#deployCronSettings : deploy cron settings from crontab file we have created before#
####################################################################################
sub deployCronSettings() {
	
	my $rc;
	my $rv;
	my @cmd;

	&printMsg("DEBUG","Updating cron tab with new settings.");

	#See if we need sudo command
	if ( $UID == 0 ) {
		if ( -e "$OPTS->{'installdir'}/crontab.org" ) {
			&printMsg("DEBUG","Found previously created crontab configuration file ... Skipping backup creation...");
			$cmd[0] = 'true';
		} else {
			&printMsg("DEBUG","Creating current crontab configuration backup.");
			$cmd[0] = "crontab -l >$OPTS->{'installdir'}/crontab.org 2>/dev/null;";
		}
                $cmd[1] = "sudo crontab -r;";
		$cmd[2] = "sudo crontab $OPTS->{'installdir'}/crontab;";
		$cmd[3] = 'sudo crontab -l';
	} else {
		if ( -e "$OPTS->{'installdir'}/crontab.org" ) {
			&printMsg("DEBUG","Found previously created crontab configuration file ... Skipping backup creation...");
			$cmd[0] = 'true';
                } else {
			&printMsg("DEBUG","Creating current crontab configuration backup.");
			$cmd[0] = "sudo crontab -l >$OPTS->{'installdir'}/crontab.org 2>/dev/null;";
		}
                $cmd[1] = "sudo crontab -r;";
		$cmd[2] = "sudo crontab $OPTS->{'installdir'}/crontab;";
		$cmd[3] = 'sudo crontab -l';
	}

	my $i;
	for($i=0; $i<=$#cmd; $i++) {
		$rv = qx($cmd[$i] 2>&1);
		$rc = $?;

		if ( $rc != 0 ) {
			&printMsg("ERROR","crontab update failed with error [$rv]");
			return 1;
		}
	}

	return 0;
}

##################################################################################################
#deployLogSettings : deploy log settings from tmp files we have created before                   #
#                    logrotated.messages.tmp --> /etc/logrotate.d/messages                       #
#                    01-threatstop-rsyslog.conf.tmp --> /etc/rsyslog.d/01-threatstop-rsyslog.conf#
#                    threatstop-logrotate.conf --> /etc/logrotate.d/threatstop-logrotate.conf    #
#                    $TMP_LOGFILE1 = logrotated.messages.tmp                                     #
#                    $TMP_LOGFILE2 = 01-threatstop-rsyslog.conf.tmp                              #
#                    $TMP_LOGFILE3 = threatstop-logrotate.conf.tmp                               #
##################################################################################################
sub deployLogSettings() {

	my $rc;
        my $rv;
        my $cmd;
	my $cmd1;
	my $cmd2;

	if ( $OPTS->{'osver'} < $OS_VER_TRIGGER ) {
		&printMsg("DEBUG","Copying modified messages -> /etc/logrotate.d/messages");

		$cmd = "mv -f $OPTS->{'installdir'}/$TMP_LOGFILE1 /etc/logrotate.d/messages";
		$cmd = "sudo $cmd" if ( $UID != 0 );
		$rv = qx($cmd 2>&1);
		$rc = $?;

		if ( $rc != 0 ) {
			&printMsg("ERROR","logratate messages deploy failed with error [$rv]");
			return 1;
		}
	} else {
		&printMsg("DEBUG","Copying modified logfile to $OPTS->{'logfile'}");

		#Now we are ready to push new files to relevant places
		#Need to be superuser to be able to do so
		if ( $UID != 0 ) {
			$cmd1 = "sudo ";
			$cmd2 = "sudo ";
		}

		#mv 01-threatstop-rsyslog.conf.tmp --> /etc/rsyslog.d/01-threatstop-rsyslog.conf
		&printMsg("DEBUG","Moving $TMP_LOGFILE2 -> $LOGRSYSLOG");
		$cmd1 .= "mv $TMP_LOGFILE2 $LOGRSYSLOG";

		#mv threatstop-logrotate.conf --> /etc/logrotate.d/threatstop-logrotate.conf
		&printMsg("DEBUG","Moving $TMP_LOGFILE3 -> $LOGSYSFILE");
		$cmd2 .= "mv $TMP_LOGFILE3 $LOGSYSFILE";

		my $rc1 = qx($cmd1 2>&1);
		my $rv1 = $?;

		if ( $rv1 != 0 ) {
			&printMsg("ERROR","Setting up [$OPTS->{'logsys'}] file failed with error [$rc1]");
			return 1;
		}

		my $rc2 = qx($cmd2 2>&1);
		my $rv2 = $?;

		if ( $rv2 != 0 ) {
			&printMsg("ERROR","Setting up [$OPTS->{'logfile'}] failed with error [${rc2}]");
			return 1;
		}
	}

	return 0;
}

########################################################################
#deployRCSettings : deploy rc.logcal file --> /etc/rc.local            #
########################################################################
sub deployRCSettings() {

	my $rc;
        my $rv;
        my $cmd;

	&printMsg("DEBUG","Copying modified rc.local -> /etc/rc.local");

	if ( $ORGRCLOCAL eq '1' ) {
		&printMsg("DEBUG","rc.local file is already up-to-date ... Skipping ...");
		return 0;
	}

	$cmd = "mv -f $OPTS->{'installdir'}/rc.local /etc; chmod +x /etc/rc.local";
	$cmd = "sudo $cmd" if ( $UID != 0 );
	
	$rv = qx(sudo $cmd 2>&1);
	$rc = $?;

	if ( $rc != 0 ) {
		&printMsg("ERROR","rc.local deploy failed with error [$rv]");
		return 1;
	}

	return 0;
}

####################################################
sub applyChanges() {
	my $cfgbackup = "$MYDIR/threatstop_preinstall_vytatta_config_backup";
	if (! -f $cfgbackup ) {
		&printMsg("INFO","We are saving your current vyatta config into $cfgbackup");
		&printMsg("INFO","You can load it back by typing load $cfgbackup inside of configure");
		&printMsg("INFO","or just use it as a reference if you want to be sure of the changes made by this script.");
		qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper begin);
		qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper save $cfgbackup);
		qx(/opt/vyatta/sbin/vyatta-cfg-cmd-wrapper end);
	}
	if ( ! defined $OPTS->{'silent'} ) {

		print "\n";
		print "---------------------------------------------\n";
		print "ATTENTION : Ready to deploy configurations !!\n";
		print "---------------------------------------------\n";
		print "Deploy (Y/N)?: [Y] ";
	
		my $opt = <STDIN>;
		chomp($opt);
		print "\n";

		if ( $opt =~ /[Nn]/ ) {
			print "Exiting setup ...\n";
			return 1;
		}
	}

	$APPLY = 1;

	system("$OPTS->{'ipset'} -L | grep $OPTS->{'setprefix'}allownet >/dev/null 2>&1");	
	if($? == 0 and $OPTS->{'type'} ne "u") {
		&printMsg("ERROR","It appears you have already configured ThreatStop on this device.");
		&printMsg("ERROR","Only untouched is supported for re-running setup.pl.");
		&printMsg("ERROR","Please run revert.sh first before selecting router or bridge setup.");
		exit(1);
	}
	&printMsg("INFO","Deploying ... ");
        deployLogMonitoringVyatta();
	return 1 if ( &generateCronJob() != 0);
	return 1 if ( &deployCronSettings() != 0 );

	if ( defined $OPTS->{'merge'}) {
		return 1 if ( &deployMergeSettings() != 0 );
	}

	if ( $ORGRCLOCAL eq '' ) {
		return 1 if ( &deployRCSettings() != 0 );
        }

	return 0;
}

#####################################################################
#getLists: running ipsetget.pl script, getting allow/block lists    #
#####################################################################
sub getLists() {

	my $cmd;
	my $rv;

	&printMsg("INFO","Getting allow/block lists.");
	
	$cmd = "$OPTS->{'src'}/ipsetget.pl";
	$cmd .= " DEBUG" if ( $DEBUG_LEVEL > 5);
	

	#silent mode - always run ipset get
	if ( ! defined $OPTS->{'silent'}) {
		print "\n";
		print "--------------------------------------\n";
		print "NOTE : Ready to run allow/block script\n";
		print "       for the first time             \n";
		print "--------------------------------------\n";
		print "Get allow/block lists now (Y/N) [Y] : ";

		my $opt = <STDIN>;
		chomp($opt);

		print "\n";
		return 0 if ( $opt =~ /[Nn]/ );
	}

	&printMsg("DEBUG","Running - $cmd");
	$rv = qx(sudo $cmd 2>&1);

        if ( $rv =~ /\[ERROR\]/ ) {
                &printMsg("ERROR","Getting allow/block lists failed.");
                return 1;
        }

	#output ipsetget.pl execution flow
	if ( ! defined $OPTS->{'silent'} ) {
		&printMsg("DISPL","");
		&printMsg("DISPL","--------------------------------------");
		&printMsg("DISPL","$rv");
		&printMsg("DISPL","--------------------------------------");
		&printMsg("DISPL","");
	} else {
		#We want to anyway log the script output 
		&logMsg("DISPL","--------------------------------------");
		&logMsg("DISPL","$rv");
		&logMsg("DISPL","--------------------------------------");
	}

	&printMsg("INFO","Finished getting allow/block lists.");
	#system("cat /tmp/tsoutput.txt") if ( ! defined $OPTS->{'silent'});
	
	return 0;
}
########################################################################
########################## MAIN PART ###################################
########################################################################

system("/usr/bin/clear"); 

print "\n";
&getCurrentTime(\$CTIME);
&logInit($SETUPLOG);
&printMsg("INFO","Starting ThreatSTOP setup operation v$VERSION at $CTIME");

&abortExec() if ( &verifyUID()               != 0 ); #verdify under what user we are running
&setLock();

&abortExec() if ( &setGID()                  != 0 ); #make sure we are in the vyattacfg group.
&abortExec() if ( &setWorkingDir()           != 0 ); #Make sure we are running from local directory
&abortExec() if ( &loadCMDData()             != 0 ); #load data from CMD supplied paramters
&abortExec() if ( &setCMDdataPRovidedPArams()!= 0 ); #set additional params based on CMD data
&abortExec() if ( &setInstallDir()           != 0 ); #set install dir if it is different than src
&abortExec() if ( &loadSRVData()             != 0 ); #SRV updates need to get in place before we try to check anything
&abortExec() if ( &loadPrevConfData()        != 0 ); #load previous data from threatstop.conf
&abortExec() if ( &loadAdditionalSettings()  != 0 ); #load additional data system settings 
&abortExec() if ( &printMsg1()               != 0 ); #display first info message
# GS-1368 (remove next line; no more need)
# &abortExec() if ( &determineHashType()       != 0 ); #ipset hash type : hash:ip or hash:net
&abortExec() if ( &verifyUploadedSettings()  != 0 ); #Check if everything is ok
&abortExec() if ( &generateConfFile()        != 0 ); #Create threatstop.conf file
&abortExec() if ( &testingConnection()       != 0 ); #Verify access to Threatstop DNS servers 
&abortExec() if ( &generateRules()           != 0 ); #Generate rules set according to install type
&abortExec() if ( &generateRClocal()         != 0 ); #Generate copy of rc.local
&abortExec() if ( &applyChanges()            != 0 ); #Apply changes...Deploy configuration files
&abortExec() if ( &getLists()                != 0 ); #Get allow/block lists

&setUnLock();
&getCurrentTime(\$CTIME);
&calculateElapsedTime($START_EPOCH,\$ETIME);
&printMsg("INFO","Finished ThreatSTOP setup operation at $CTIME after $ETIME"); 
&printMsg2() if ( $APPLY == 0);

print "\n";

