#!/usr/bin/perl
#
# Copyright(c) 2006-2017 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

use strict;
use warnings;
use diagnostics;
use File::Basename;
use File::Copy;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/lib/";
use ThreatStop;

my $MYPATH  = $FindBin::Bin;
chomp (my $VERSION = `cat $MYPATH/VERSION`);
my $CONFILE = "${MYPATH}/threatstop.conf";
my $CFG={}; # global config data
my $CTIME;
my $LCKFILE = $0 . ".lck";
my @CHANGED_CFGS = ();
$DEBUG_LEVEL = 6 if ( defined $ARGV[0] && $ARGV[0] eq "DEBUG" );

##################################################################
# loadSRVData - Values from the SRV record will take precedence
# over settings from the config file.  This gives DNS-data-driven
# control over external services.
#
# For the DNS server settings, we'll use the SRV in favor of
# what's there.  NOTE, the port will not be influenced by the
# SRV record.
#
# For the log upload target, we'll adjust the URL according to
# what is retrieved.
#
# We expect only SRV for the service/protocol queried; if more
# that one is returned, use the first we encounter.
#
# global data : $CFG, the global config hash reference
# uses : ThreatStop::querySRV()
#
# returns :
# non-zero - if any SRV targets are different from what was already in the $CFG
# 0 - if we cannot get an SRV
sub loadSRVData {

    my $domain = 'threatstop.com';
    my $changed = 0;

    # DNS server
    my $srvref = [];
    my $service = 'tsdns';
    my $protocol = 'tcp';
    # this should be able to run prior to the system probe for utilities,
    # but we'll favor the CFG version, if it's already there
    chomp (my $dig = defined $CFG->{dig} ? $CFG->{dig} : qx(which dig));

    if ( ThreatStop::querySRV( $dig, $service, $protocol, $domain, $srvref ) ) {

        my $ref = $$srvref[0]; # getting here, this should be available
        &printMsg("DEBUG","Comparing '$CFG->{dns_server}' with '$ref->{target}'");
        if ( $CFG->{dns_server} ne $ref->{target} ) {

            &printMsg("DEBUG","noting new 'dns_server' => '$ref->{target}'");
            $CFG->{dns_server} = $ref->{target};
            push @CHANGED_CFGS, 'dns_server';
        }
    } else {

        &printMsg("DEBUG", "No SRV found for $service, $protocol, $domain");
        &printMsg("DEBUG", "Will not change from $CFG->{dns_server}");
    }

    # log upload URL
    $srvref = [];
    $service = 'logs';
    $protocol = 'https';

    if ( ThreatStop::querySRV( $dig, $service, $protocol, $domain, $srvref ) ) {

        my $ref = $$srvref[0]; # getting here, this should be available

        # SRV records return trailing dots on the target, which can cause issues
        # with some web clients pulling a url that has a trailing dot in the host
        if ( $ref->{target} =~ /\.$/ ) {

            chop $ref->{target};
        }

        my $url = "$protocol://$ref->{target}:$ref->{port}/cgi-bin/logupload.pl";
        &printMsg("DEBUG","Comparing '$CFG->{url}' with '$url'");
        if ( $CFG->{url} ne $url ) {

            &printMsg("DEBUG","noting new 'url' => '$url'");
            $CFG->{url} = $url;
            push @CHANGED_CFGS, 'url';
        }
    } else {

        &printMsg("DEBUG", "No SRV found for $service, $protocol, $domain");
        &printMsg("DEBUG", "Will not change from $CFG->{url}");
    }

    return 0;
}

# loads $CFG hash
# returns
# 1 - success
# 0 - not
sub loadCFGData {

    my $err = '';
    my $rval = 1;
    
    &printMsg("INFO", "Loading configuration");

    if ( -e $ThreatStop::CONFIGFILE ) {

        if ( &loadConfFileData($ThreatStop::CONFIGFILE, $CFG, \$err) != 0 ) {

            &printMsg("ERROR", $err);
            $rval = 0;
        }
    } else {

        &printMsg("WARN", "Configuration file does not exist; initial installation?");
        $rval = 0;
    }

    return $rval;
}

# Update changed configuration values.
# The configuration file is assumed to exist.
#
# uses :
# - global @CHANGED_CFGS
# - $ThreatStop::CONFIGFILE - config file name
#
# returns :
# 1 - success
# 0 - problem opening configuration file
sub updateCFG {

    return 1 if ! scalar @CHANGED_CFGS; # the common case, post initial-installation

    if ( ! open(CFG, "<$ThreatStop::CONFIGFILE") ) {

        &printMsg("ERROR", $!);
        return 0;
    }
    my @cfgLines = <CFG>;
    close CFG;

    if ( ! open(CFGNEW, ">$ThreatStop::CONFIGFILE.new") ) {

        &printMsg("ERROR", $!);
        return 0;
    }

    my $wroteCfgLine;
    foreach my $cfgLine ( @cfgLines ) {

        $wroteCfgLine = 0;
        foreach my $cfg ( @CHANGED_CFGS ) {

            if ( $cfgLine =~ /^$cfg=/ ) {

                print CFGNEW qq|$cfg="$CFG->{$cfg}"\n|;
                $wroteCfgLine = 1;
            }
        }

        print CFGNEW $cfgLine if ! $wroteCfgLine;
    }
    close CFGNEW;

    if ( ! copy("$ThreatStop::CONFIGFILE.new", $ThreatStop::CONFIGFILE) ) {

        &printMsg("ERROR", $!);
        return 0;
    }

    return 1;
}

# last breath
sub _myDie {

    my $dying_statement = shift;

    &printMsg("ERROR", $dying_statement);
    exit 1;
}

########################################################################
# MAIN
&getCurrentTime(\$CTIME);
&printMsg("INFO","Starting ThreatSTOP service updates v$VERSION at $CTIME");
setLock();
loadCFGData() or _myDie("Unable to load configuration");
loadSRVData(); # will only update what's new
updateCFG() or _myDie("Unable to update the configuration");

# lose lock
END {
    &setUnLock(); 
    qx(rm -f $LCKFILE) if ( -e $LCKFILE );
}


